@@ -59,7 +59,6 @@
 #include "password_dialog.h"
 #include "monitor.h"
 #include "remove.h"
-#include "trash.h"
 #include "widgets.h"
 
 enum
@@ -269,7 +268,7 @@ static void move2trash(char *path)
 	globfree(&dirlist);
     }
 #endif
-    add2trash(path);
+    TRASH_add2trash(path);
 #ifdef DEBUG
 	printf("DBG2:move2trash %s\n",path); 
 #endif
