@@ -1,4 +1,4 @@
-/*  $Id: item_dialog.c,v 1.71 2004/09/19 09:29:16 jasper Exp $
+/*  $Id: item_dialog.c 4331 2005-02-16 20:06:02Z jasper $
  *
  *  Copyright (C) 2002-2004 Jasper Huijsmans (jasper@xfce.org)
  *
@@ -48,7 +48,7 @@
 #include "settings.h"
 
 #define BORDER          6
-#define PREVIEW_SIZE    48
+#define PREVIEW_SIZE    32
 
 typedef struct _ItemDialog ItemDialog;
 
@@ -80,18 +80,19 @@ static GtkWidget *menudialog = NULL;	/* keep track of this for signal
 
 static GtkTargetEntry entry[] = {
     {"text/uri-list", 0, 0},
-    {"STRING", 0, 1}
+    {"UTF8_STRING", 0, 1},
+    {"STRING", 0, 2}
 };
 
 static const char *keys[] = {
     "Name",
+    "GenericName",
     "Comment",
     "Icon",
     "Categories",
     "OnlyShowIn",
     "Exec",
-    "Terminal",
-    NULL
+    "Terminal"
 };
 
 /* useful widgets */
@@ -193,15 +194,15 @@ drag_drop_cb (GtkWidget * widget, GdkDragContext * context, gint x,
     gchar *temp = NULL;
     gchar *buf = NULL;
     gchar *exec = NULL;
-    gchar **execp = NULL;
+    gchar *execp = NULL;
     XfceDesktopEntry *dentry;
 
     if (sd->data)
     {
 	if (g_str_has_prefix (sd->data, "file://"))
-	    buf = g_build_filename (&(sd->data)[7], NULL);
+	    buf = g_strdup (&(sd->data)[7]);
 	else if (g_str_has_prefix (sd->data, "file:"))
-	    buf = g_build_filename (&(sd->data)[5], NULL);
+	    buf = g_strdup (&(sd->data)[5]);
 	else
 	    buf = g_strdup (sd->data);
 
@@ -218,24 +219,27 @@ drag_drop_cb (GtkWidget * widget, GdkDragContext * context, gint x,
 	if (g_file_test (filename, G_FILE_TEST_EXISTS) &&
 	    XFCE_IS_DESKTOP_ENTRY (dentry =
 				   xfce_desktop_entry_new (filename, keys,
-							   7)))
+                                       G_N_ELEMENTS (keys))))
 	{
-
-	    if (xfce_desktop_entry_get_string (dentry, "Exec", FALSE, &exec)
-		&& exec)
-	    {
-		if (g_strrstr (exec, "%") != NULL)
-		{
-		    execp = g_strsplit (exec, "%", 0);
-		    g_free (exec);
-		    exec = g_strdup (execp[0]);
-		    g_strfreev (execp);
-		}
-		command_options_set_command (opts, exec, FALSE, FALSE);
-	    }
-
-	    g_object_unref (dentry);
+            xfce_desktop_entry_get_string (dentry, "Exec", FALSE, &exec);
+            g_object_unref (dentry);
+            g_free (filename);
 	}
+        else
+        {
+            exec = filename;
+        }
+
+        if (exec)
+        {
+            if ((execp = g_strrstr (exec, " %")) != NULL)
+            {
+                execp[0] = '\0';
+            }
+            
+            command_options_set_command (opts, exec, FALSE, FALSE);
+            g_free (exec);
+        }
     }
 
     gtk_drag_finish (context, sd->data != NULL, FALSE, time);
@@ -443,7 +447,7 @@ update_icon_preview (int id, const char *path, IconOptions * opts)
     if (id == EXTERN_ICON)
     {
         if (path)
-            pb = gdk_pixbuf_new_from_file (path, NULL);
+            pb = xfce_themed_icon_load (path, PREVIEW_SIZE);
     }
     else
     {
@@ -548,7 +552,7 @@ icon_browse_cb (GtkWidget * w, IconOptions * opts)
 
     text = gtk_entry_get_text (GTK_ENTRY (opts->icon_entry));
 
-    file = select_file_with_preview (_("Select command"), text,
+    file = select_file_with_preview (NULL, text,
 				     gtk_widget_get_toplevel (opts->base));
 
     if (file && g_file_test (file, G_FILE_TEST_EXISTS) &&
@@ -767,6 +771,24 @@ G_MODULE_EXPORT /* EXPORT:icon_options_set_icon */
 void
 icon_options_set_icon (IconOptions * opts, int id, const char *path)
 {
+    const char *icon_path = NULL;
+
+    if (opts->icon_id == EXTERN_ICON)
+        icon_path = gtk_entry_get_text (GTK_ENTRY (opts->icon_entry));
+
+    if (id == opts->icon_id)
+    {
+        if (id != EXTERN_ICON)
+        {
+            return;
+        }
+        else if (path && icon_path && 
+                 strlen(icon_path) && !strcmp(path,icon_path))
+        {
+            return;
+        }
+    }
+    
     g_signal_handler_block (opts->icon_menu, opts->id_sig);
     gtk_option_menu_set_history (GTK_OPTION_MENU (opts->icon_menu),
 				 (id == EXTERN_ICON) ? 0 : id);
@@ -784,11 +806,6 @@ icon_options_set_icon (IconOptions * opts, int id, const char *path)
     }
     else
     {
-	const char *icon_path = NULL;
-
-	if (opts->icon_id == EXTERN_ICON)
-	    icon_path = gtk_entry_get_text (GTK_ENTRY (opts->icon_entry));
-
 	if (icon_path && strlen (icon_path))
 	{
 	    g_free (opts->saved_path);
@@ -875,6 +892,10 @@ static inline void
 add_icon_options (GtkBox * box, ItemDialog * idlg, GtkSizeGroup * sg)
 {
     idlg->icon_opts = create_icon_options (sg, TRUE);
+    
+    /* TODO: This check should probably be in item.c */
+    if (idlg->item->icon_id == 0)
+        idlg->item->icon_id = EXTERN_ICON;
 
     icon_options_set_icon (idlg->icon_opts, idlg->item->icon_id,
 			   idlg->item->icon_path);
@@ -1182,7 +1203,7 @@ G_MODULE_EXPORT /* EXPORT:edit_menu_item_dialog */
 void
 edit_menu_item_dialog (Item * mi)
 {
-    GtkWidget *remove, *close, *header;
+    GtkWidget *remove, *close, *header, *vbox;
     GtkDialog *dlg;
     int response, num_items;
 
@@ -1228,8 +1249,13 @@ edit_menu_item_dialog (Item * mi)
 	add_spacer (GTK_BOX (dlg->vbox), BORDER);
     }
 
+    vbox = gtk_vbox_new (FALSE, BORDER);
+    gtk_widget_show (vbox);
+    gtk_container_set_border_width (GTK_CONTAINER (vbox), BORDER - 1);
+    gtk_box_pack_start (GTK_BOX (dlg->vbox), vbox, FALSE, FALSE, 0);
+    
     /* add the other options */
-    create_item_dialog (NULL, mi, GTK_CONTAINER (dlg->vbox), close);
+    create_item_dialog (NULL, mi, GTK_CONTAINER (vbox), close);
 
     add_spacer (GTK_BOX (dlg->vbox), BORDER);
 
@@ -1254,6 +1280,8 @@ edit_menu_item_dialog (Item * mi)
 
     gtk_widget_destroy (menudialog);
 
+    menudialog = NULL;
+
     write_panel_config ();
 }
 
