@@ -620,7 +620,7 @@ static void setup_dialog(Itf *itf)
     g_signal_connect(G_OBJECT(itf->checkblink), "toggled", G_CALLBACK(cb_checkblink_changed), itf);
     g_signal_connect(G_OBJECT(itf->hscale3), "value_changed", (GCallback) cb_blinktime_changed, itf);
 
-    gtk_window_set_position(GTK_WINDOW(itf->keyboard_dialog), GTK_WIN_POS_CENTER);
+    xfce_gtk_window_center_on_monitor_with_pointer (GTK_WINDOW (itf->keyboard_dialog));
     gtk_widget_show(itf->keyboard_dialog);
 }
 
@@ -823,22 +823,26 @@ create_channel(McsPlugin *mcs_plugin)
 static gboolean
 write_options(McsPlugin *mcs_plugin)
 {
-    gboolean result;
+    gboolean result = FALSE;
     gchar *rcfile, *path;
 
     path = g_build_filename ("xfce4", RCDIR, RCFILE1, NULL);
     rcfile = xfce_resource_save_location (XFCE_RESOURCE_CONFIG, path, TRUE);
-    result = mcs_manager_save_channel_to_file(mcs_plugin->manager, CHANNEL1,
-		    rcfile);
-    g_free(path);
-    g_free(rcfile);
+    if (G_LIKELY (rcfile != NULL))
+    {
+        result = mcs_manager_save_channel_to_file (mcs_plugin->manager, CHANNEL1, rcfile);
+        g_free (rcfile);
+    }
+    g_free (path);
 
     path = g_build_filename ("xfce4", RCDIR, RCFILE2, NULL);
     rcfile = xfce_resource_save_location (XFCE_RESOURCE_CONFIG, path, TRUE);
-    result &= mcs_manager_save_channel_to_file(mcs_plugin->manager, CHANNEL2,
-		    rcfile);
-    g_free(path);
-    g_free(rcfile);
+    if (G_LIKELY (rcfile != NULL))
+    {
+        result &= mcs_manager_save_channel_to_file (mcs_plugin->manager, CHANNEL2, rcfile);
+        g_free (rcfile);
+    }
+    g_free (path);
 
     return(result);
 }
