@@ -74,7 +74,6 @@
 #include "remove.h"
 #include "keybindings.h"
 #include "easy.h"
-#include "xfstab.h"
 
 /*XXX funky smartbutton for column (alpha):*/
 #include "toolbar.h"
@@ -87,8 +86,8 @@
 #define LOCAL_OVERRIDE_MASK (SHOW_MM|SHOW_TB1|SHOW_TB2|SHOW_F|SHOW_TITLES|RSH_X_SSH|RSYNC_X_SCP)
 
 
+extern gboolean enable_pane_resize;
 
-extern gchar *bookfile;
 
 
 /* keep order the same as defined for enum at libs/types.h: */
@@ -114,6 +113,77 @@ void menu_detach(void)
     /*printf ("dbg:menu_detach()\n"); */
 }
 
+static
+gboolean    on_name_size (GtkWidget *widget, GtkAllocation *allocation, gpointer user_data){
+    int which = (int)((long)user_data);
+    if (!enable_pane_resize) return FALSE;
+    /*printf("DBG %d is changing...%d\n",which,widget->allocation.width);*/
+    tree_details->treestuff[which].name_size=widget->allocation.width;
+    return TRUE;
+}
+
+static
+gboolean    on_size (GtkWidget *widget, GtkAllocation *allocation, gpointer user_data){
+    static int pane_pos;
+    
+    GtkWidget *hpaned;
+
+    if (!enable_pane_resize) {
+#ifdef DEBUG
+	printf("pane resize not enabled\n");
+#endif
+	return FALSE;
+    }
+    hpaned = WIDGET("hpaned1");
+#ifdef DEBUG
+    printf("DBG:pane width=%d\n",hpaned->allocation.width);
+#endif
+    pane_pos=gtk_paned_get_position((GtkPaned *)hpaned);
+    if (tree_details->geometryX == tree_details->window->allocation.width &&
+	tree_details->geometryY == tree_details->window->allocation.height &&
+	tree_details->hpane != gtk_paned_get_position((GtkPaned *)hpaned)
+	){
+        tree_details->hpane = gtk_paned_get_position((GtkPaned *)hpaned);
+	tree_details->hpane_ratio = (double)tree_details->hpane / (double)hpaned->allocation.width;
+	set_widget_initial_state();
+	turn_on();
+#ifdef DEBUG
+	printf("pane size is changing... new ratio=%lf (%d/%d)\n",
+		tree_details->hpane_ratio,tree_details->hpane,hpaned->allocation.width);
+#endif
+    } else if (tree_details->geometryX != tree_details->window->allocation.width ||
+	    tree_details->geometryY != tree_details->window->allocation.height ||
+	    (tree_details->hpane_ratio >= 0 && tree_details->hpane != tree_details->hpane_ratio * hpaned->allocation.width))
+    {
+	if (tree_details->geometryX != tree_details->window->allocation.width ||
+	    tree_details->hpane != tree_details->hpane_ratio * hpaned->allocation.width)
+	{
+	    int pane_pos = tree_details->hpane_ratio * hpaned->allocation.width;
+#ifdef DEBUG
+	    /* keep pane ratio constant*/
+	    printf("geometryX is changing... ratio=%lf, setting pane pos to %d\n",
+		tree_details->hpane_ratio,pane_pos);
+#endif
+	    gtk_paned_set_position(GTK_PANED(hpaned), pane_pos);
+	    tree_details->hpane = gtk_paned_get_position((GtkPaned *)hpaned);
+	} else {
+	    tree_details->geometryY = tree_details->window->allocation.height;
+#ifdef DEBUG
+	    printf("geometryY change... not interesting.\n");
+#endif
+	    return TRUE;
+	}
+	tree_details->geometryX = tree_details->window->allocation.width;
+	tree_details->geometryY = tree_details->window->allocation.height;	    
+    } else {
+#ifdef DEBUG
+	printf("size allocation event for main window...\n");
+#endif
+	return TRUE;
+    }
+    write_xffm_config(); 
+    return TRUE;
+}
 
 
 
@@ -166,21 +236,38 @@ void create_root_element(GtkTreeView * treeview, GtkTreeIter *iterator,
 	    en->count = -1;
 	    set_row_colours(treemodel, iterator, NULL,DIR_COLOR);
 	}
-	else if(i == ROOT_BOOKMARKS && bookfile)
+	else if(i == ROOT_BOOKMARKS)
 	{
-	    name = my_utf_string((const gchar *)bookfile);
-	    /* this is bugged: en->path=g_strdup(bookfile);
-	     * en->path for bookmarks operations should not be used in other modules
-	     * to eliminate this bug */
-	    en->path = g_strdup(_(root[i].path));	    
+	    gchar *bookfile=NULL;
+	    TRACE ("doing %s %s",tree_details->argv[0],tree_details->argv[1]);
+	    if (strstr(tree_details->argv[0],"xfbook") && tree_details->argv[1]) {
+		bookfile = g_path_get_basename(tree_details->argv[1]);
+		TRACE("bookfile is %s",bookfile);
+		if (bookfile && strlen(bookfile)){
+		    DBG("using %s\n",bookfile);
+		}
+		/* XXX: bookfile path saved as utf-8 */
+		g_free(tag);
+		tag = g_strdup(my_utf_string((const gchar *)bookfile));
+		TRACE("name is %s",tag);
+		BOOK_set_bookfile((const gchar *)tag);
+		g_free(bookfile);
+		/* this is not allowed here: en->path=g_strdup(bookfile);*/
+	    }
+	    name = tag;
+	    TRACE("2 name is %s",name);
+	    en->path = g_strdup(_(root[i].path));
+	    
 	}
 	else 
 	{
 	    name = tag;
 	    en->path = g_strdup(_(root[i].path));
+	    TRACE("3 name is %s",name);
 	}
 	/* icon size set by config-file */
 /* FIXME: should be "basic" function:*/
+	TRACE("setting name to %s",name);
 	gtk_tree_store_set((GtkTreeStore *) treemodel, iterator, 
 			ENTRY_COLUMN, en, 
 			NAME_COLUMN, g_strdup(name), 
@@ -596,7 +683,7 @@ static GtkTreeView *create_treeview(char *path,gint tree_id)
 /* if it fails to set directory, it goes to root,
  * if no directory is given in argv, it goes to home */
 G_MODULE_EXPORT
-GtkTreeView *init_xffm(int argc, char *argv[])
+GtkTreeView *init_xffm_treeviews(int argc, char *argv[])
 {
     int i;
     gchar *startup=NULL;
@@ -604,6 +691,7 @@ GtkTreeView *init_xffm(int argc, char *argv[])
 #ifdef DEBUG
     printf("DBG:tree_details->argv[0]=%s\n",tree_details->argv[0]);
 #endif
+
     if((strstr(tree_details->argv[0],"xffm") || strstr(tree_details->argv[0],"xftree"))&& argc >= 2)
     {				/* if it fails it goes to root. */
 	if(argv[1][0] == '/')
@@ -623,40 +711,9 @@ GtkTreeView *init_xffm(int argc, char *argv[])
 	    }
 	}
     }
-#ifdef DEBUG
-    printf("startup is %s\n",((startup)?startup:"NULL"));
-#endif
+    DBG("startup is %s, tmp=%s\n",((startup)?startup:"NULL"),g_get_tmp_dir());
     chdir(g_get_tmp_dir());
 
-    parse_colours();
-    /* set in settings... tree_details->theme =  NULL;*/
-    
-
-
-    if((tree_details->window = create_xffm()) == NULL)	assert_not_reached();
-    /*XXX: this should be removed when the glade code generation 
-     * is removed (if ever)*/
-    g_signal_connect(G_OBJECT(tree_details->window), 
-		    "destroy_event", G_CALLBACK(on_xffm_destroy_event), (gpointer) tree_details);
-    g_signal_connect(G_OBJECT(tree_details->window), 
-		    "delete_event", G_CALLBACK(on_xffm_delete_event), (gpointer) tree_details);
-    hideit(tree_details->window, "item22");
-    hide_text(tree_details->window);
-    hideit(tree_details->window, "stop");
-    hideit(tree_details->window, "input_box");
-    /* XXX does this "activates-default" do anything??? */
-    g_object_set(G_OBJECT (WIDGET("combo_entry2")), "activates-default", TRUE, NULL);
-    g_object_set(G_OBJECT (WIDGET("input_entry")), "activates-default", TRUE, NULL);
-    {
-	int i;
-	/* NOTICE: keep names[] order in sync with on_sort_activate() 
-	 * at libs/refresh.c */
-	char *names[] = { "show_hidden1","preview_images1", NULL};
-	for(i = 0; names[i]; i++){
-	    g_signal_connect(G_OBJECT(WIDGET(names[i])), "activate", 
-			    GTK_SIGNAL_FUNC(on_sort_activate), (gpointer) ((long)i));
-	}
-    }
 
    
     /*if (!startup) startup=g_strdup(GETWD);*/
@@ -673,6 +730,11 @@ GtkTreeView *init_xffm(int argc, char *argv[])
       /*printf("DBG: treeview is 0x%x\n",treeview);*/
       setup_drag_signal(treestuff->treeview);
       apply_view(treestuff->treeview);
+      g_signal_connect ((gpointer) treestuff->treeview, 
+	      "size-allocate", G_CALLBACK (on_size), NULL);
+      g_signal_connect ((gpointer) (treestuff->column[NAME_COLUMN]->button),
+	      "size-allocate", G_CALLBACK (on_name_size), 
+	      (gpointer)((long)i));
     }
     g_free(startup);startup=NULL;
 
