@@ -46,13 +46,6 @@
 
 #include "icon-theme.h"
 
-#ifdef DEBUG
-#undef DEBUG
-#endif
-/*#define DEBUG*/
-/*#define DD(x) x*/
-#define DD(x) 
-
 /*extern GtkWidget *main_window;*/
 
 static GHashTable *theme_hash = NULL;
@@ -99,7 +92,7 @@ static xfmime_icon_functions *xfmime_icon_fun=NULL;
 G_MODULE_EXPORT
 const gchar *g_module_check_init(GModule *module){
     xfmime_icon_fun = g_new0 (xfmime_icon_functions,1);
-    D(printf("g_module_check_init...\n");)
+    TRACE("g_module_check_init...\n");
     if (!xfmime_icon_fun) return ("unable to create function structure");
     xfmime_icon_fun->mime_icon_find_pixmap_file =mime_icon_find_pixmap_file ;
     xfmime_icon_fun->mime_icon_get_iconset = mime_icon_get_iconset;
@@ -174,10 +167,10 @@ gchar *mime_icon_get_theme_path(const gchar *theme){
     xfce_resource_pop_path(XFCE_RESOURCE_ICONS);
     
     /* literal name */
-    D(printf("mime_icon_get_theme_path...%s\n",theme);)
+    TRACE("mime_icon_get_theme_path...%s\n",theme);
     for (p=sdirs;*p;p++){
 	g=g_build_filename(G_DIR_SEPARATOR_S,*p,theme,NULL);
-    D(printf("mime_icon_get_theme_path...%s?\n",g);)
+    TRACE("mime_icon_get_theme_path...%s?\n",g);
 	if (g_file_test(g,G_FILE_TEST_IS_DIR)) {
 	    goto done;
 	}
@@ -186,7 +179,7 @@ gchar *mime_icon_get_theme_path(const gchar *theme){
     
     /* literal name not found: is it a translated name? */
     
-    D(printf("mime_icon_get_theme_path...2\n");)
+    TRACE("mime_icon_get_theme_path...2\n");
     for (p=sdirs;*p;p++)
     {
         XfceRc *themefile;
@@ -194,15 +187,15 @@ gchar *mime_icon_get_theme_path(const gchar *theme){
 	const gchar *themename;
 	GDir *dir;
 	const gchar *file;
-    D(printf("mime_icon_get_theme_path...2.0:%s\n",*p);)
+    TRACE("mime_icon_get_theme_path...2.0:%s\n",*p);
 
 	if (!g_file_test(*p,G_FILE_TEST_IS_DIR)) continue;
     
         dir = g_dir_open(*p, 0, NULL);
-    D(printf("mime_icon_get_theme_path...2.1\n");)
+    TRACE("mime_icon_get_theme_path...2.1\n");
         if(!dir) continue;
 
-    D(printf("mime_icon_get_theme_path...%s\n",*p);)
+    TRACE("mime_icon_get_theme_path...%s\n",*p);
         while((file = g_dir_read_name(dir)))
         {
             themeindex = g_build_path(G_DIR_SEPARATOR_S, *p, file, "index.theme", NULL);
@@ -233,7 +226,7 @@ gchar *mime_icon_get_theme_path(const gchar *theme){
     }
 done:
     g_strfreev(sdirs);  
-    D(printf("mime_icon_get_global_xml_file...done\n");)
+    TRACE("mime_icon_get_global_xml_file...done\n");
     return g;
 }
 
@@ -244,7 +237,7 @@ static gchar **mime_icon_get_inherits(const gchar *theme){
     theme_path=mime_icon_get_theme_path(theme);
     if (!theme) return NULL;
      
-	DD(printf("DBG: inherit themepath=%s\n",theme_path);)
+	TRACE("inherit themepath=%s\n",theme_path);
        
     /* inherited paths... */
 
@@ -316,7 +309,7 @@ skip_duplicate:;
    len = g_list_length(list);
    dirs = g_new0(char *, len + 1);
    for(i = 0, li = list; li; li = li->next, i++) {
-	/*printf("DBG:got %s\n",(char *)li->data);*/
+	/*TRACE("got %s\n",(char *)li->data);*/
 	dirs[i] = (char *)li->data;
    }
    g_list_free(list);
@@ -366,9 +359,7 @@ static void start_element (GMarkupParseContext *context,
 	      /*printf("hashing %s=%s\n",name,icon);*/
 	  }
 	  else if (name) {	      
-#ifdef DEBUG
-	      printf("no icon defined for %s\n",name);
-#endif
+	      DBG("no icon defined for %s\n",name);
 	      g_free(name);
 	  }
 
@@ -415,7 +406,7 @@ mime_icon_get_global_xml_file(const gchar *theme_name){
 	g_warning("theme_name==NULL");
 	return NULL;
     }
-    D(printf("mime_icon_get_global_xml_file name...%s\n",theme_name);)
+    TRACE("mime_icon_get_global_xml_file name...%s\n",theme_name);
     themepath=mime_icon_get_theme_path(theme_name);
     if (themepath) {
 	gchar *b=g_path_get_basename(themepath);
@@ -425,13 +416,13 @@ mime_icon_get_global_xml_file(const gchar *theme_name){
 	g_free(b);
     }
      
-    DD(printf("mime_icon_get_global_xml_file...3:%s\n",mimefile);)
+    TRACE("mime_icon_get_global_xml_file...3:%s\n",mimefile);
    /* test for default mimefile (xfce distributed)  */
    if (!mimefile || !g_file_test(mimefile,G_FILE_TEST_EXISTS)) {
         gchar **inherits;
         gchar **p;
 	
-	DD(printf("DBG: global file %s not found\n",mimefile);)
+	TRACE("global file %s not found\n",mimefile);
         g_free(mimefile); mimefile=NULL;
 	inherits = mime_icon_get_inherits(theme_name);
 	/* test for inherited default mimefiles (xfce distributed) */
@@ -439,7 +430,7 @@ mime_icon_get_global_xml_file(const gchar *theme_name){
 	    mimefile=g_strconcat(PACKAGE_DATA_DIR,G_DIR_SEPARATOR_S,
 	        "xfce4",G_DIR_SEPARATOR_S,"mime",G_DIR_SEPARATOR_S,
 		 *p,".mime.xml",NULL);
-	    DD(printf("DBG: inherits file %s \n",mimefile);)
+	    TRACE("inherits file %s \n",mimefile);
 	    if (g_file_test(mimefile,G_FILE_TEST_EXISTS)) break;
 	    g_free(mimefile); mimefile=NULL;
 	}
@@ -464,7 +455,7 @@ static int create_icon_hash(void){
 
     theme_info_t *theme_info;  
 
-    D(printf("create_icon_hash...\n");)
+    TRACE("create_icon_hash...\n");
 
     /*g_free(icon_theme_name);*/
 
@@ -482,8 +473,9 @@ static int create_icon_hash(void){
     /*printf("hey: iconthemename=%s\n",icon_theme_name);*/
 
 
-    if (open_theme(icon_theme_name)==-1) {
-	D(printf("icon_hash already exists...\n");)
+    /* we specify the icon size to use the best match cache */
+    if (open_theme(icon_theme_name, 48)==-1) {
+	TRACE("icon_hash already exists...\n");
 	return -1;
     }
     
@@ -491,13 +483,13 @@ static int create_icon_hash(void){
 
    /* test for user defined mimefile (via xfmime-edit)*/
    mimefile= mime_icon_get_local_xml_file(icon_theme_name);
-    D(printf("create_icon_hash from local...%s\n",mimefile);)
+    TRACE("create_icon_hash from local...%s\n",mimefile);
 
    /* test for default mimefile (xfce distributed)  */
    if (!mimefile || !g_file_test(mimefile,G_FILE_TEST_EXISTS)) {
        g_free(mimefile);
 	mimefile=mime_icon_get_global_xml_file(icon_theme_name);
-      D(printf("create_icon_hash from global ...3\n");)
+      TRACE("create_icon_hash from global ...3\n");
    }
    
    if (!mimefile || !g_file_test(mimefile,G_FILE_TEST_EXISTS)) {
@@ -508,13 +500,13 @@ static int create_icon_hash(void){
    }
    
     if (theme_hash==NULL) {
-	    D(printf("DBG:creating new theme_hash...\n");)
+	    TRACE("creating new theme_hash...\n");
 	    theme_hash = g_hash_table_new(g_str_hash, g_str_equal);
     }
 
     theme_info = g_hash_table_lookup(theme_hash,icon_theme_name);
     if (!theme_info){
-        D(printf("creating new icon hash and factory for %s\n",icon_theme_name);)
+        TRACE("creating new icon hash and factory for %s\n",icon_theme_name);
 	theme_info = (theme_info_t *)malloc(sizeof(theme_info_t));
     	theme_info->icon_hash = icon_hash = g_hash_table_new(g_str_hash, g_str_equal);
     	theme_info->icon_factory = icon_factory = gtk_icon_factory_new();
@@ -531,7 +523,7 @@ static int create_icon_hash(void){
    
     g_free(mimefile);mimefile=NULL;
 	    
-    D(printf("DBG:theme_hash created!\n");)
+    TRACE("theme_hash created!\n");
     return 1;
 }
 
@@ -547,7 +539,7 @@ GdkPixbuf *mime_icon_create_pixbuf                          (const gchar     *fi
   const gchar *fullpath;
   if (!filename || !filename[0]) return NULL;
   if ((fullpath = find_icon_path(filename,48,"MimeType"))==NULL) return NULL;
-  /*printf("DBG: creating pixbuf for... %s\n",fullpath);*/
+  /*TRACE("creating pixbuf for... %s\n",fullpath);*/
   pixbuf = xfce_pixbuf_new_from_file_at_size(fullpath, 48,48,&error);
   if (error) g_error_free(error);
   return pixbuf;
@@ -566,9 +558,7 @@ GtkWidget *mime_icon_create_pixmap (GtkWidget *widget, const gchar     *filename
 
   if (!pathname)
     {
-#ifdef DEBUG
-      g_warning ("Couldn't find pixmap file: %s", filename);
-#endif
+      DBG ("Couldn't find pixmap file: %s", filename);
       return gtk_image_new ();
     }
   pixmap = gtk_image_new_from_file (pathname);
@@ -582,13 +572,11 @@ GtkWidget *mime_icon_create_pixmap (GtkWidget *widget, const gchar     *filename
 G_MODULE_EXPORT
 GtkIconSet *mime_icon_get_iconset(const gchar *id, GtkWidget *main_window){
     GtkIconSet *icon_set;
-    gchar *iconfile;
+    const gchar *iconfile;
     GdkPixbuf *pixbuf;
     gchar *gg=NULL;
 
-#ifdef DEBUG
-    printf("looking for %s in theme\n",id);
-#endif
+    TRACE("looking for %s in theme",id);
     if (!id || !strlen(id)) return NULL;
 
     if (!icon_hash) return NULL;
@@ -596,8 +584,14 @@ GtkIconSet *mime_icon_get_iconset(const gchar *id, GtkWidget *main_window){
     if (icon_set) return icon_set;
 
     /* is the id an actual path? (custom icons) */
-    if (access(id,F_OK)==0) iconfile=(gchar *)id;
-    else iconfile=g_hash_table_lookup(icon_hash,id);
+    if (access(id,F_OK)==0) iconfile=id;
+    else {
+	const gchar *path=mime_icon_find_pixmap_file(id);
+	TRACE("path is %s \n",path);
+	if (path) iconfile=path;
+	else iconfile=(const gchar *) g_hash_table_lookup(icon_hash,id);
+    }
+    
     if (!iconfile || !strlen(iconfile)) {
 	    if (strchr(id,'/')){
 		    gchar *g=g_strdup(id);
@@ -613,29 +607,26 @@ GtkIconSet *mime_icon_get_iconset(const gchar *id, GtkWidget *main_window){
 	    }
     }
     if (!iconfile || !strlen(iconfile)) return NULL;
-#ifdef DEBUG
-    printf("DBG:adding icon to iconfactory: %s->%s\n",id,iconfile);
-#endif
+    DBG("adding icon to iconfactory: %s->%s",id,iconfile);
 
     if(strncmp("gtk-", iconfile, strlen("gtk-")) == 0){
-#ifdef DEBUG
-    	printf("DBG:rendering icon %s...\n",iconfile);
-#endif
+    	DBG("rendering icon %s...\n",iconfile);
  	pixbuf = gtk_widget_render_icon(main_window, iconfile, GTK_ICON_SIZE_DIALOG, NULL);
-#ifdef DEBUG
-    	printf("DBG: %s rendered.\n",iconfile);
-#endif
+    	DBG("%s rendered.\n",iconfile);
     }
     else {
 	/* full path specified? (as in custom icons) */    
-	if (access(id,F_OK)==0) {
+	if (access(iconfile,F_OK)==0) {
 	   GError *error=NULL;
-	   pixbuf = gdk_pixbuf_new_from_file (id, &error);
+	   pixbuf = gdk_pixbuf_new_from_file (iconfile, &error);
 	   if (error) g_error_free(error);
 	   if (!pixbuf) return NULL;
 	}
 	else {
 	    pixbuf=mime_icon_create_pixbuf(iconfile);
+	    if (!pixbuf){
+		g_warning("problem with %s",iconfile);
+	    }
 	}
     }
 	
@@ -674,10 +665,10 @@ void g_module_unload(GModule *module){
 G_MODULE_EXPORT
 int mime_icon_load_theme(void){
     int r;
-    D(printf("mime_icon_load_theme...\n");)
+    TRACE("mime_icon_load_theme...\n");
     r=create_icon_hash();
     if (r) return r;
-    g_message("cannot create icon_hash");
+    g_warning("cannot create icon_hash");
     return r;
 }
 
